// A trivial (but nevertheless, working quite well) nonogram solver,
// based on binary decision diagrams (BDD).
//
// For more information on nonograms (or Paint-by-Number Puzzle) see e.g.
// http://webpbn.com/
// 
// The numbered programs are sequential parts of presentation, showing 
// different stages of solver development. The first fully working nonogram
// solver is in nonogrambdd2.cpp.
//
// This particular file demonstrates generation of weak integer compositions.
// 
// The program requires BuDDy (libbdd-dev package on Debian). Supposing BuDDy
// is installed system-wide, compilation can be done with the command: 
// g++ -o nonogrambdd1 -lbdd -O2 nonogrambdd0.cpp
//                                                                          
// (c) 2011 Konstantin L. Metlov <metlov@fti.dn.ua>                         
//                                                                          
// You can redistribute and/or modify this file under the terms of the GNU  
// General Public License as published by the Free Software Foundation, eith
// version 3 of the License, or (at your option) any later version.         
//                                                                          
// This file is distributed in the hope that it will be useful,             
// but WITHOUT ANY WARRANTY; without even the implied warranty of           
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            
// GNU General Public License for more details.

#include <cstdlib>                                                          
#include <cmath>                                                            
#include <cassert>
#include <iostream>
#include <iomanip>
#include <bdd.h>

using namespace std;

void comp_first(int n, int k, int* x);
bool comp_next(int n, int k, int* x);

int main(int argc, char *argv[]) { 
  // print out some compositions
  const int k=3;
  const int n=4;
  int x[k];
  comp_first(n,k,x);
  do {
    for (int i=0;i<k;i++) cout << setw(2) << x[i] << ' ';
    cout << endl;
  } while (comp_next(n,k,x));
};

// initializes compisition generator to enumerate compositions of
// n into k parts with the first composition returned in the array x
// (must be of the length k).
void comp_first(int n, int k, int* x) {
  x[0]=n;
  for(int i=1;i<k;i++) x[i]=0; 
};

// generates the next composition into the array x. returns false if the
// composition in the array x is already lexicographically the last one.
bool comp_next(int n, int k, int* x) {
  int s=0;
  for (int i=0; i<k; i++) {
    assert(x[i]>=0);
    s+=x[i];
  };
  assert(s==n);
  if (x[k-1]==n) return false;
  int i=0;
  while(x[i]==0) i++; // find the first non-zero element
  int v=x[i];
  x[i]=0;
  x[0]=v-1;
  x[i+1]++;
  return true;
};
